/*
 * Copyright (C) 1992  Board of Regents of the University of Wisconsin
 * on behalf of the Department of Electrical Engineering and Computer
 * Science, University of Wisconsin-Milwaukee, Milwaukee, WI 53201.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The programs in this directory were developed by software engineering
 * teams as part of the course "Introduction to Software Engineering"
 * under the supervision of Professor G. Davida.
 *
 * Please send all changes, enhancements, and other comments about this
 * software to
 *
 *     		soft-eng@cs.uwm.edu
 *
 *			or
 *		
 *		Software Engineering Coordinator
 *		Computer Science
 *   		Department of EECS
 *		University of Wisconsin - Milwaukee
 *		Milwaukee, WI  53201
 *		414-229-4677
 */
/* $Log:	display_all.c,v $
 * Revision 1.3  93/01/18  22:40:50  jimj
 * Completely removed the Xw widgets
 * And now the code looks better
 * 
 * Revision 1.2  92/12/31  23:46:37  jimj
 * Fixed some minor cleanups
 * 
 * Revision 1.1  1992/09/24  23:47:05  jimj
 * Initial revision
 *
 */

#include <sys/types.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/List.h>

#define ALL_LENGTH     80  /* Length of each line in the top section */
/*  Stuff from other files */

extern int numpacketslist;	/*  Number of packets in list */
extern char *data_all[];	/*  The array of packets      */
extern int data_list[];		/*  Contains number of which packets to view*/
extern whatis_general();	/*  Figure out what packet is */
static int selection;		/*  Points to the currently selected packet */
static Widget alledit, headeredit;
void draw_all(), change_selection();

static XtActionsRec actionsTable[] = {
     {"change-selection", change_selection},
};

/**  This procedure opens up the window and the scrollbar etc that 
 **  is associated with the window on top that displays all the
 **  packets in a general form
 **/

void create_display_all(parent)
Widget parent;
{
  /* static XawTextSelectType selarray[] = {XawselectLine, XawselectNull}; */
   Arg wargs[7];

   XtAddActions(actionsTable, XtNumber(actionsTable));
   XtSetArg(wargs[0], XtNtype, XawAsciiString);
   XtSetArg(wargs[1], XtNscrollVertical, XawtextScrollAlways);
   XtSetArg(wargs[2], XtNwrap, XawtextWrapLine);
   XtSetArg(wargs[3], XtNdisplayCaret, FALSE);
   XtSetArg(wargs[4], XtNstring, 
      "     Destination            Source         Type     Source     Dest");

   alledit = XtCreateManagedWidget("headeredit", asciiTextWidgetClass, 
                                   parent, wargs, 5);

   XtSetArg(wargs[3], XtNdisplayCaret, TRUE);
   XtSetArg(wargs[4], XtNstring, "James Jegers   jimj@cs.uwm.edu");
/*   XtSetArg(wargs[5], XtNselectTypes, (XtArgVal) selarray);  */

   alledit = XtCreateManagedWidget("alledit", asciiTextWidgetClass, 
                                   parent, wargs, 5);
}

void change_selection(w, event, params, num_params)
   Widget w;
   XEvent *event;
   String *params;
   int num_params;
{
   Arg wargs[1];
   int oldselection = selection;

   if (!strcmp(params[0], "down")) 
       selection++;

   if (!strcmp(params[0], "up")) 
       selection--;

   if (!strcmp(params[0], "select"))
       selection = XawTextGetInsertionPoint(alledit) / (ALL_LENGTH + 1);

   if (selection < 0) 
       selection = 0;
   if (selection >= numpacketslist )
       selection = numpacketslist - 1;

/*  If nothing changed don't go updating everything, that flicker
 *  is annoying, takes the case when you go past the total existing packets
 */
   if (oldselection != selection)	
   {
       XawTextSetInsertionPoint(alledit, selection * (ALL_LENGTH + 1) );

       draw_detail(data_list[selection]); /*  Redraw the display detail part */
       draw_hex(data_list[selection]);	  /*  Redraw the display hex part    */
   }
} 

/** If things change figure out what the new
 ** things in the window would be
 **/

void draw_all()
{
   Arg wargs[2];
   char alldata[5000];
   char string[ALL_LENGTH + 5];
   int index = 0;

/**  Get all the info about the packet **/

   strcpy(alldata, "");
   while (index < numpacketslist) {
	whatis_general(string, data_all[data_list[index]]);
	stringpad(string, ' ', ALL_LENGTH);
	strcat(alldata, string);
	strcat(alldata, "\n");
	index++; 
   } 

/** Update the actual widget **/
   XtSetArg(wargs[0], XtNstring, alldata);
   XtSetValues(alledit, wargs, 1);

/** Change all the other widgets with the new selection **/
   selection = 0;
   XawTextSetInsertionPoint(alledit, 0);
   draw_detail(data_list[selection]);	/*  Redraw the display detail part */
   draw_hex(data_list[selection]);	/*  Redraw the display hex part    */
}

