/*
 * Copyright (C) 1992  Board of Regents of the University of Wisconsin
 * on behalf of the Department of Electrical Engineering and Computer
 * Science, University of Wisconsin-Milwaukee, Milwaukee, WI 53201.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The programs in this directory were developed by software engineering
 * teams as part of the course "Introduction to Software Engineering"
 * under the supervision of Professor G. Davida.
 *
 * Please send all changes, enhancements, and other comments about this
 * software to
 *
 *     		soft-eng@cs.uwm.edu
 *
 *			or
 *		
 *		Software Engineering Coordinator
 *		Computer Science
 *   		Department of EECS
 *		University of Wisconsin - Milwaukee
 *		Milwaukee, WI  53201
 *		414-229-4677
 */

/* $Log:	types.c,v $
 * Revision 1.2  92/12/31  23:55:22  jimj
 * 
 * Revision 1.1  1992/09/24  23:51:42  jimj
 * Initial revision
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include "names.h"
#include "types.h"

char *sizeit();

/*
 * Convert an address in the format of  my.cs.uwm.edu  to  129.89.9.115
 */
my2ip(string, address)
char *string;
char *address;
{
	char temp[30];

	sprintf(temp, "%d.%d.%d.%d", (int) strtol(address, (char **) NULL,16),
	(int) strtol(&address[3], (char **) NULL, 16),
	(int) strtol(&address[6], (char **) NULL, 16),
	(int) strtol(&address[9], (char **) NULL, 16)  );
	strcpy(string, temp);
}

/*  Convert a ethernet number address to the real name in human form */

ether2real(string, ethernumber)
char *string;
char ethernumber[];
{
	int loop;
	char hardware[20];

		/*  Take the address in form  11 22 33 44  and move to new
		 *  string which we can modify
		 */
	for (loop = 0; loop <= 17; loop++) 
    	    hardware[loop] = ethernumber[loop];
	hardware[17] = '\0';

		/*  Put the :'s between the bytes */
	hardware[2] = hardware[5] = hardware[8] = 
				   hardware[11] = hardware[14] = ':';

		/*  Loop though all the spots until we find the address
		 *  In our table
		 */
	loop = 0;
	while ((name[loop].ether[0] != 'N') && 
	       (strcmp(name[loop].ether, hardware)))
    	   loop++;

	      /* if we quit and not at the end. most have found 1 */
   	if(name[loop].ether[0] != 'N') 
			/* Convert the address to real name */
		ip2real(string, name[loop].ip);
   	else 
			/*  Couldn't find name, just return hardware address */
		strcat(string, hardware);

} /* realmachine */

/* Convert an ip number in packet list form to the real name
 * eg, input will be  c5 d3 09 15  converts to 129.89.9.15 or something
 * like that.  It then addes the real name to the end of that in
 * parenthesis
 */
#define hash_size 5000 
#define hash_mask 4999 

struct host_ip_ent {
	int h_address;
	int empty;
	char h_name[30];
	struct host_ip_ent *h_nxt;
};
typedef struct host_ip_ent host_ip_ent;
static host_ip_ent *hash_table[hash_size];

	/*  Initiaze the table */
void init_ip2real()
{
	register int i;
	for (i=0; i<hash_size; i++)
		hash_table[i]=NULL;
}
	
	/*  Convert an ip number to a real name...  */
ip2real(retaddress, address)
char *retaddress, address[];
{
	host_ip_ent *ptr;
	host_ip_ent *temp;
	unsigned int index;
	struct hostent * host;
	int h_address;

	strcat(retaddress, "  (");

		/*  Get the address in 4byte int form */
	h_address = inet_addr(address);
		/*  compute index, by mod of mask */
	index = ((unsigned int) h_address % hash_mask);

		/*  Look in my table to see if we can find it */
	for(ptr = hash_table[index]; ptr != NULL; ptr=ptr->h_nxt) {

			/*  If found name, return the name */
		if (ptr->h_address == h_address) {
			strcat(retaddress, ptr->h_name);
			strcat(retaddress, ")");
			return;
		}
	}
		/*  If name not in table */
	temp=hash_table[index];
	ptr=hash_table[index]=(host_ip_ent *)malloc(sizeof(host_ip_ent));
	ptr->h_nxt = temp;
	ptr->h_address = h_address;

		/*  See if we can figure out what the host name is */
	if((host = gethostbyaddr( (char *) &h_address, sizeof(int), AF_INET)) != NULL) 
		strcpy(ptr->h_name, sizeit(host->h_name, 15) );
	else 
		strcpy(ptr->h_name, sizeit(address, 15) );
	
	strcat(retaddress, ptr->h_name);
	strcat(retaddress, ")");

} /* convert ip number to real name */

	/*  Figure what type of packet it is.. IP or ARP */

ether_type(retstring, packet)
char *retstring, packet[];
{
 	char string[20];

	string[0] = packet[0]; string[1] = packet[1];
	string[2] = packet[3]; string[3] = packet[4];
	string[4] = '\0';

	if(!strcmp(string, "0800")) {
		strcat(retstring, "   IP");
		return(IP);
	} 
	if(!strcmp(string, "0806")) {
		strcat(retstring, "  ARP");
		return(ARP);
	}

	strcat(retstring, string);
	return(OTHER);
} /*  ether type */

	/*  Figure out what type of IP packet it is */

ip_type(retstring, packet)
char *retstring, *packet;
{
	int i;

	i = (int) strtol(packet, (char **) NULL, 16);

	if (i == 6) {
		strcat(retstring, " TCP");
		return(TCP);
	}
	if (i == 1) {
		strcat(retstring, " ICMP");
		return(ICMP);
	}
	if (i == 12){
		strcat(retstring, " PUP");
	 	return(PUP);
	}
	if (i == 17){
		strcat(retstring, " UDP");
	 	return(UDP);
	}

	strcat(retstring, " OTHER");
	return(OTHER);
} /* ip type */

	/*  Look at the port number and figure out what the port is  */

port_type(retstring, packet)
char *retstring;
char *packet[];
{
	char string[10];
	int loop;

	string[0] = '\0';

	packet2int(string, packet, 4);

	switch(atoi(string)) {
	case 20:
		strcat(retstring, "ftp-data");
		break;	
	case 21:
		strcat(retstring, "     ftp");
		break;
	case 23:
		strcat(retstring, "  telnet");
		break;
	case 25:
		strcat(retstring, "    smtp");
		break;
	case 513:
		strcat(retstring, "  rlogin");
		break;
	case 514:
		strcat(retstring, "   shell");
		break;
	default:
			/*  Then just copy the number of the port */
		for(loop =0; loop < 8 -  strlen(string); loop++)
		    strcat(retstring, " ");
		strcat(retstring, string);
		break;
	}
}

init_checkether()
{
int addr, spot, newspot;
char temp[30];

	for (addr = 0; name[addr].ether[0] != 'N'; addr++) {
		newspot = 0;
		for(spot = 0; spot < strlen(name[addr].ether); spot++) {
			if (name[addr].ether[spot+1] == ':' || 
			    name[addr].ether[spot+1] == '\0' ) {
				temp[newspot++] = '0';
				temp[newspot++] = name[addr].ether[spot];
				temp[newspot++] =  ':';
				spot++;
			} else {
				temp[newspot++] = name[addr].ether[spot];
				temp[newspot++] = name[addr].ether[spot+1];
				temp[newspot++] = ':';
				spot += 2;
			}
		}
		temp[newspot-1] = '\0';
		strcpy(name[addr].ether, temp);
	}
}
