/*
 * Copyright (C) 1992  Board of Regents of the University of Wisconsin
 * on behalf of the Department of Electrical Engineering and Computer
 * Science, University of Wisconsin-Milwaukee, Milwaukee, WI 53201.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The programs in this directory were developed by software engineering
 * teams as part of the course "Introduction to Software Engineering"
 * under the supervision of Professor G. Davida.
 *
 * Please send all changes, enhancements, and other comments about this
 * software to
 *
 *     		soft-eng@cs.uwm.edu
 *
 *			or
 *		
 *		Software Engineering Coordinator
 *		Computer Science
 *   		Department of EECS
 *		University of Wisconsin - Milwaukee
 *		Milwaukee, WI  53201
 *		414-229-4677
 */

#include <X11/Xlib.h>
#include "defs.h"

extern Display *mydisplay;
extern Window mywindow;
extern GC mygc;
extern GC mygcblank;

/*
 *  Draw the up-down bar graphs for the packet size 
 */
bargraph(newvalue, oldvalue, offset)
int newvalue, oldvalue, offset;
{
  int units, units2;

  if (newvalue >= oldvalue) {

  	/*  Make the bar bigger by drawing a new one of needed size */
     units = (float) newvalue / SAMPLES * 159;
     XFillRectangle(mydisplay, mywindow, mygc, 
		    50+(26*offset), 200 - units, 20, units);

  } else {

     	/* Make bar smaller, by erasing the extra part from before */
     units = (float) oldvalue / SAMPLES * 159 ;
     units2 = (float) (oldvalue - newvalue) / SAMPLES * 159;
     XFillRectangle(mydisplay, mywindow, mygcblank,
 		    50+(26*offset), 200 - units - 1, 20, units2 + 1);
  
  }  /* if then else */

}  /* Bargraph */


/* 
 *  Bar graph for the network load line 
 */ 
loadbargraph(newvalue, oldvalue)
float newvalue, oldvalue;
{
   if (newvalue >= oldvalue) 

   	/*  Draw a new bar of bigger size */

       XFillRectangle(mydisplay, mywindow, mygc, 41, 251,
       		      (int) (newvalue * 249), 39);

   else
   	/*  Erase the extra part to make the bar smaller */

       XFillRectangle(mydisplay, mywindow, mygcblank, 
                      41 + (int)(newvalue * 249), 251, 
       		      (int) ((oldvalue - newvalue) * 249 + 1), 39);

}  /* load bar graph */

/* 
 *  Bar graph for the type of packets
 */

typebargraph(newvalue, oldvalue, offset)
int newvalue, oldvalue, offset;
{
   if (newvalue >= oldvalue) {

   		/*  Make the bar bigger by redrawing */
 	XFillRectangle(mydisplay, mywindow, mygc,  351, 50 + 14*offset, 
	               (int) ( (float) (newvalue) / SAMPLES * 119), 10);
   } else {

   		/*  Make smaller by eraseing part of it */

	XFillRectangle(mydisplay, mywindow, mygcblank, 
	   	   351 + (int) ( (float) (newvalue) / SAMPLES * 119), 
	   	   50 + 14*offset,
	          (int) ((oldvalue - newvalue) / (float) (SAMPLES) * 119 + 1),
	           10);
   } /* if */

}  /* type bar graph */

static int spot = 0;

loadovertime(load)
float load;
{

	 	  /*  Draw a vertical load bar */

	XDrawLine(mydisplay, mywindow, mygc, 
	          spot+351, 219, spot+351, 219 - (int) (load*89)  );

	spot++;   /* Increment the spot where the line is drawn */

		  /*  If next line is outside of box, reset it  */
	if (spot >= 120) {
  	    spot = 0;
  	    XClearArea(mydisplay, mywindow, 351, 131, 119, 89, False);
	}

} /* load over time bar graphs */

