/*
 * Copyright (C) 1992  Board of Regents of the University of Wisconsin
 * on behalf of the Department of Electrical Engineering and Computer
 * Science, University of Wisconsin-Milwaukee, Milwaukee, WI 53201.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The programs in this directory were developed by software engineering
 * teams as part of the course "Introduction to Software Engineering"
 * under the supervision of Professor G. Davida.
 *
 * Please send all changes, enhancements, and other comments about this
 * software to
 *
 *     		soft-eng@cs.uwm.edu
 *
 *			or
 *		
 *		Software Engineering Coordinator
 *		Computer Science
 *   		Department of EECS
 *		University of Wisconsin - Milwaukee
 *		Milwaukee, WI  53201
 *		414-229-4677
 */

#include <stdio.h>
#include <sys/file.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

Display *mydisplay;
Window mywindow;
GC mygc;
GC mygcblank;
char *fontname = "6x13";
int filedes;

/*  Main function to set up the terminal and
 *  set up the ethernet card for reading
 */
 
main(argc, argv)
int argc;
char *argv[];
{
    int myscreen;
    unsigned long myforeground, mybackground;
    XSizeHints myhint;
    XFontStruct *font_info;
    extern char *optarg;
    extern int optind;
    int option;
    char *usedisplay = NULL;
    char *filter = NULL;

#ifdef ultrix
#define FLAGS "d:f:"
#else
#define FLAGS "d:"
#endif

	while ((option = getopt(argc, argv, FLAGS)) != -1)
		switch(option) {
		case 'd':
			usedisplay = optarg;
	    		break;
#ifdef ultrix
		case 'f':
			filter = optarg;
			break;
#endif
	}

#ifdef ultrix
#define HAVEPF
	if (optind < argc) {
	    printf("Ethernet graph program\n");
	    printf("Usage: [-d displayname] [-f filter name] \n");
	    printf("Filter name in form of  pf0,pf1,etc \n");
	    exit(2);
	}
#endif
#ifdef sun
#define HAVEPF
	if (optind != argc - 1) {
	    printf("Ethernet graph program\n");
	    printf("Usage: [-d displayname] \n");
	    exit(2);
	}
	else
	   filter = argv[optind];
#endif

#ifdef HAVEPF
	filedes = openethercard(filter);
#else
	fprintf(stderr, "Sorry this program only runs under Ultrix or SunOS\n");
	fprintf(stderr, "Using stdin as input\n");
	filedes = 0;
#endif	

/* Now that the ethernet card is open, pop up a window 
 * This next part makes a connection to the xterm
 * and opens up a window.  It then calls xmenu to allow
 * the user to pick which option from the main menu
 */

	mydisplay = XOpenDisplay(usedisplay);
	if(!mydisplay) {
   	    fprintf(stderr, "Ahhhhhh!!!!  Cannot open display.\n");
   	    exit(1);
	}
	myscreen = DefaultScreen(mydisplay);
	mybackground = WhitePixel(mydisplay, myscreen);
	myforeground = BlackPixel(mydisplay, myscreen);
	myhint.x = 100; myhint.y = 100; myhint.width=500, myhint.height=500;
	myhint.flags = PPosition | PSize;
	mywindow = XCreateSimpleWindow(mydisplay, DefaultRootWindow(mydisplay),
		myhint.x, myhint.y, myhint.width, myhint.height, 5,
		myforeground, mybackground);
	XSetStandardProperties(mydisplay, mywindow, "X Network Monitor",
		"X Network Monitor", None, argv, argc, &myhint);
	mygc = XCreateGC(mydisplay, mywindow, 0,0);
	mygcblank = XCreateGC(mydisplay, mywindow, 0,0);
	load_font(&font_info);
	set_font(font_info);
	XSetBackground(mydisplay, mygc, mybackground);
	XSetBackground(mydisplay, mygcblank, mybackground);
	XSetForeground(mydisplay, mygc, myforeground);
	XSetForeground(mydisplay, mygcblank, mybackground);
	XSelectInput(mydisplay, mywindow, ButtonPressMask | ExposureMask);
	XMapRaised(mydisplay, mywindow);

/*
 *  Main loop to get input and display the information
 */

	basicwindow();

/*
 *  Close everything up and exit nicely
 */

	XFreeGC(mydisplay, mygc);
	XDestroyWindow(mydisplay, mywindow);
	XCloseDisplay(mydisplay);
	close(filedes);

} /*  Main program  */


/* 
 *  A couple support routines for the main program
 *  to do fonts and such
 */

load_font(font_info)
XFontStruct **font_info;
{
	if ((*font_info = XLoadQueryFont(mydisplay, fontname)) == NULL){
           fprintf(stderr, "Cann't open font %s\n", fontname);
           exit(-1);
 	}
}

set_font(font_info)
XFontStruct *font_info;
{
	XSetFont(mydisplay, mygc, font_info->fid);
}

