/*
 * Copyright (C) 1992  Board of Regents of the University of Wisconsin
 * on behalf of the Department of Electrical Engineering and Computer
 * Science, University of Wisconsin-Milwaukee, Milwaukee, WI 53201.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The programs in this directory were developed by software engineering
 * teams as part of the course "Introduction to Software Engineering"
 * under the supervision of Professor G. Davida.
 *
 * Please send all changes, enhancements, and other comments about this
 * software to
 *
 *     		soft-eng@cs.uwm.edu
 *
 *			or
 *		
 *		Software Engineering Coordinator
 *		Computer Science
 *   		Department of EECS
 *		University of Wisconsin - Milwaukee
 *		Milwaukee, WI  53201
 *		414-229-4677
 */


#include <stdio.h>
#include <sys/file.h>
#include <time.h>		/*  Ultrix's packet filter wants it */
#include "defs.h"

#ifdef ultrix

#include <net/pfilt.h>
#define MAXSIZETOTAKE 14

  struct enfilter f = {
  140, 0, ENF_PUSHONE | ENF_COR
  };


/*
 *   Ethernet card stuff  *
 */
int openethercard(device)
char *device;
{
int filedes;

int MaxBacklog = MAXBACKLOG;
int trunc = MAXSIZETOTAKE;
u_short ClearBits =  ENBATCH | ENCOPYALL;
u_short SetBits =  ENPROMISC | ENNONEXCL | ENTSTAMP; 

	filedes = pfopen(device, O_RDONLY);
	if (filedes < 0) {
    	     perror("Error opening pf device");
    	     exit(1);
	}
	if(ioctl(filedes, EIOCSETF, &f) == -1 ) {
    	    perror("Error setting priority");
    	    exit(1);
	}
	if(ioctl(filedes, EIOCMBIC, &ClearBits) == -1) {
    	    perror("Error setting stuff to clear");
    	    exit(1);
	}
	if (ioctl(filedes, EIOCMBIS, &SetBits) == -1 ) {
   	    perror("Error setting stuff to set");
    	    exit(1);
	}
	if(ioctl(filedes, EIOCSETW, &MaxBacklog) == -1 ) {
    	    perror("Could not set backlog");
    	    exit(1);
	};
	ioctl(filedes, EIOCTRUNCATE, &trunc);

	ioctl(filedes, EIOCFLUSH, 0);

	return(filedes);
} /* open ethernet card */

#endif ultrix

#ifdef sun
#include <fcntl.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/stropts.h>
#include <net/nit.h>
#include <net/if.h>
#include <net/nit_if.h>
#include <net/nit_pf.h>
#include <net/packetfilt.h>

#include <netinet/in.h>
#include <netinet/if_ether.h>

int openethercard(device)
char *device;
{
  struct strioctl si;
  struct ifreq ifr;
  struct ether_header eh;
  struct packetfilt pf;
  register u_short *fwp;
  int if_fd;

  /*  Open NIT device */
  if ((if_fd = open("/dev/nit", O_RDWR)) == -1) {
     perror("Cann't open /dev/nit");
     exit(1);
  }

     /*
      * Arrange to get message-discard from the stream
      */
     if (ioctl(if_fd, I_SRDOPT, (char *) RMSGD) < 0) {
 	perror("ioctl(I_SRDOPT) failed");
 	exit(1);
     }
#ifdef notdef
     /*
      * Push on protocol filter
      */
     if (ioctl(if_fd, I_PUSH, "pf") < 0) {
 	perror("ioctl(I_PUSH) failed");
 	exit(1);
     }
     /*
      * Filter only ARP requests
      */
     fwp = pf.Pf_Filter;
     *fwp++ = ENF_PUSHWORD + ((u_int) & eh.ether_type -
 			     (u_int) & eh.ether_dhost) / sizeof(u_short);
     *fwp++ = ENF_PUSHLIT | ENF_EQ;
     *fwp++ = htons(ETHERTYPE_ARP);
     pf.Pf_FilterLen = fwp - pf.Pf_Filter;
     pf.Pf_Priority = 5;
     si.ic_cmd = NIOCSETF;
     si.ic_timout = INFTIM;
     si.ic_len = sizeof(pf);
     si.ic_dp = (char *) &pf;
     if (ioctl(if_fd, I_STR, (char *) &si) < 0) {
 	perror("ioctl(I_STR,NIOCSETF) failed");
 	exit(1);
     }
#endif
     /*
      * Bind NIT stream
      */
     (void) strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name) - 1);
     ifr.ifr_name[sizeof(ifr.ifr_name) - 1] = '\0';
     si.ic_cmd = NIOCBIND;
     si.ic_timout = INFTIM;
     si.ic_len = sizeof(ifr);
     si.ic_dp = (char *) &ifr;
     if (ioctl(if_fd, I_STR, (char *) &si) < 0) {
 	perror("ioctl(I_STR,NIOCBIND) failed");
 	exit(1);
     }
     return (if_fd);
 }
#endif
