/*
 * Copyright (C) 1992  Board of Regents of the University of Wisconsin
 * on behalf of the Department of Electrical Engineering and Computer
 * Science, University of Wisconsin-Milwaukee, Milwaukee, WI 53201.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The programs in this directory were developed by software engineering
 * teams as part of the course "Introduction to Software Engineering"
 * under the supervision of Professor G. Davida.
 *
 * Please send all changes, enhancements, and other comments about this
 * software to
 *
 *     		soft-eng@cs.uwm.edu
 *
 *			or
 *		
 *		Software Engineering Coordinator
 *		Computer Science
 *   		Department of EECS
 *		University of Wisconsin - Milwaukee
 *		Milwaukee, WI  53201
 *		414-229-4677
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Label.h>
#include <stdio.h>
#include <sys/types.h>
#include <time.h>		/*  Ultrix's packet filter wants it */

#include "defs.h"

#ifdef ultrix
#include <time.h>
#include <net/pfilt.h>
#endif /* ultrix */

#define KEEPGOING FALSE
#define DONE	  TRUE

extern void get_text();
extern void get_save();
Boolean sample_data();
extern int filedes;

u_char *buffer;
u_char *bufferspot;
static XtWorkProcId work_proc_id = 0;
Widget status, statusdone;

void stop_sample_data()
{
    Arg wargs[1];

    		/*  If the thing is sampleing data stop it  */
    if(work_proc_id) {
	XtRemoveWorkProc(work_proc_id);
	*bufferspot = (int) 0;
	XtSetArg(wargs[0], XtNlabel, "Saveing Data");
	XtSetValues(status, wargs, 1);
	print_data();
    }
    		/*  Reset the counters and make things look pretty */
    work_proc_id = 0;
    XtSetArg(wargs[0], XtNlabel, "Ready");
    XtSetValues(status, wargs, 1);
    XtSetArg(wargs[0], XtNlabel, "  0.0%");
    XtSetValues(statusdone, wargs, 1);
}

int do1, do2;
char *goodata[6];
void start_sample_data()
{
    Arg wargs[1];

    	/*  Make a buffer which we can store the data into  */
    buffer = (u_char *) malloc(BUFFERSIZE);
    bufferspot = buffer;

    	/*  Update the screen */
    XtSetArg(wargs[0], XtNlabel, "Sampleing Data");
    XtSetValues(status, wargs, 1);

    	/*  Find out what packets the user wanted  */
    get_text(goodata);

    	/* Something in the frist and second options section */
    do1 = goodata[FROM1][0] || goodata[TO1][0];
    do2 = goodata[FROM2][0] || goodata[TO2][0];

#ifdef ultrix
    	/*  Flush the kernels queue of packets  */
    ioctl(filedes, EIOCFLUSH, 0);
#endif /* ultrix */

    	/*  Tell Xt to run my routine  */
    if (work_proc_id)
	XtRemoveWorkProc(work_proc_id);
    work_proc_id = XtAddWorkProc(sample_data, NULL);
}

/*
 *  This procedure actually samples the data in 
 */

static u_int tper = 0;
Boolean sample_data()
{
    Arg wargs[1];
    char str[7];

    	/*  Every 200 packets update the screens percentage count */
    if (!(tper % 200)) {
         sprintf(str, "%5.1f%%", 
                (bufferspot - buffer) / (float)BUFFERSIZE * 100); 
         XtSetArg(wargs[0], XtNlabel, str);
         XtSetValues(statusdone, wargs, 1);
         tper = 0;
    }
    tper++;

   	/*  Check to see if the buffer is big enough to hold another packet */

    if ( bufferspot < &buffer[BUFFERSIZE] - 2000 ) {

	*bufferspot = (int) read(filedes, (bufferspot + sizeof(int)), 2000);

	if ( * (int *) bufferspot == EOF ) {
	     perror("Error reading from device");
	     exit(1);
 	}

	/* Check to see if a type was given.. If given it has to match.
  	 * If it doesn't match exit with a FALSE; 
         */
		/*  If a type was specifed, the type has to match  */

	if (goodata[TYPE][0] && 
	                  type(&bufferspot[12] + sizeof(int), goodata[TYPE]) )
  	     return(KEEPGOING);

		/* If nothing given for both selections then we are done */

	if(!do1 && !do2)  {
		bufferspot += *bufferspot + sizeof(int);
		tper++;
		return(KEEPGOING);
	}

		/*  If data is in the first selection section */
	if (do1) {
		if (address(bufferspot + sizeof(int),     goodata[1]) &&
       	            address(&bufferspot[6] + sizeof(int), goodata[0]) )   {
				/*  This packet good, move pointer */
			bufferspot += *bufferspot + sizeof(int);
		}
	}

		/*  If data is in the second selection section */
	if (do2) {
		if (address(bufferspot + sizeof(int),     goodata[4]) &&
	    	    address(&bufferspot[6] + sizeof(int), goodata[3]) )    {
				/*  This packet good, move pointer */
			bufferspot += *bufferspot + sizeof(int);
		}
	} /* if */

		/*  Return without moving point position, next packet will
		 *  Use the same place that this one did
		 */
	return(KEEPGOING);

	/*  Return true to Xt if the buffer is full, and signal program 
	 *  to save the data.
         */

    } else {
	stop_sample_data();
	return DONE;
    }

} /* Sample some data */


/*  
 *  Save the data that we have collected to a file. 
 */

print_data()
{
    FILE *fd;
    int size, loop;
    u_char *bufferspot;
    char name[40];

    get_save(name);
    if ((fd = fopen(name, "w")) == NULL) {
	perror("Error saving to file");
	return;
    }

    bufferspot = buffer;
    while ((int) *bufferspot != 0) {
	size = (int) *bufferspot;
	bufferspot += sizeof(int);
	for(loop = 0; loop < size; loop++) {
		fprintf(fd, "%02x ", *bufferspot);
		bufferspot++;
	}
    fprintf(fd, "\n");
    } /* while */

    fclose(fd);
} /* print */

/*
 *  Check to see if the from to TO address matches what
 *  we want it to match 
 */
address(is, want)
u_char *is, *want;
{
    char newis[20];

    	/*  If user didn't specify what he wanted, make any packet match */
    if (want[0] == 0)
	return(1);

    sprintf(newis, "%02x:%02x:%02x:%02x:%02x:%02x",
	 is[0], is[1], is[2], is[3], is[4], is[5]);

    return(!strcmp(newis, want));

} /* see which type matches */

/*
 *  Check to see if the type matches what we want
 */
type(is, want)
u_char *is, *want;
{
    char newis[6];

    	/*  If user didn't specify type he wanted, make any packet match */
    if(want[0] == 0)
	return(1);

    sprintf(newis, "%02x%02x", is[0], is[1]);

    return(!strcmp(newis, want));
}

status_line(parent)
Widget parent;
{
    Arg wargs[2];

    XtSetArg(wargs[0], XtNlabel, "Ready");
    status = XtCreateManagedWidget("Ready", labelWidgetClass, parent,
				wargs, 1);

    XtSetArg(wargs[0], XtNlabel, "  0.0%");
    statusdone = XtCreateManagedWidget("done", labelWidgetClass, parent,
				wargs, 1);
}

